extensions [imos array matrix]

;;; Lets keep things organised
breed [eyes eye]
breed [ernests ernest]
breed [targets target]
breed [markers marker]

eyes-own [previously-seen previously-smelled distance-of-target distance-of-fish] ;; Varaiables needed to encode change

directed-link-breed [left-eyes left-eye]
directed-link-breed [right-eyes right-eye]

ernests-own [status stimuli1 config_action satisfaction]

globals [rcounter ##int_targ_status ##target-color ##fish-color language list_hit list_seefood list_gofood list_eatfood sentences words counter config mat weights output]

;; called when the model is first initialised.
to startup
 learn-strategy -1 -1 1 2 3 4
end

to setup
  ca 
  clear-output
  reset-ticks 
  set rcounter 0
  set counter 0
  draw-environment
  set ##target-color pink
  set ##fish-color yellow
  set list_hit []
  set list_seefood []
  set list_gofood []
  set list_eatfood []
  set sentences []
  set language []
  set output []
  set mat matrix:make-constant 250 350 0
  set weights matrix:make-constant 250 250 0
  set words [ "hitwall" "seefood" "gofood" "eatfood"]
  
  file-open "hitwall.txt"
  while [ not file-at-end? ] [
  set list_hit lput file-read-line list_hit
  ]
  file-close
  
   file-open "seefood.txt"
  while [not file-at-end?]
  [set list_seefood lput file-read-line list_seefood
  ]
  file-close
  
   file-open "gofood.txt"
  while [not file-at-end?]
  [set list_gofood lput file-read-line list_gofood
  ]
  file-close
  
   file-open "eatfood.txt"
  while [not file-at-end?]
  [set list_eatfood lput file-read-line list_eatfood
  ]
  file-close
  
  ;; Now there is an imos for each Ernest
  ;;imos:init 10 4
  ;;imos:interaction ">" "   " step
  ;;imos:interaction "^" "   " turn
  ;;imos:interaction "v" "   " turn
  create-ernest
end

;;; Create a random board and initialise Ernest.
to random-board
  setup
  place-target-random
  end

;;; Sets default values to the learning component of Ernest.
to default
  set step -1
  set bump -8
  set turn 0
  set appear 15
  set closer 10
  set disappear -15  
  setup
end

;;; Simple procedure to set ernest and the target in 
;;; prespecified places to learn a new strategy. 
;;; Learns with default values for interactions. 
to learn-strategy [ernx erny tx ty fx fy]
  default
  place-target-random
  
  ask ernests [setxy ernx erny]
  ask targets [setxy tx ty]
  end

;;; Runs N experiments without intialising parameters.
;;; This is nice to see what has ernest learned.
to safe-run-a-series
  ;;setup 
  let t 0
  if (number-of-runs <= 0) [stop]
    set t ticks
    while [any? targets ] [ifelse (ticks - t < timeout) [run-experiment] [print-timeout-message ask targets [die]]]
    wait 0.2
    clear-drawing
    draw-environment 
    place-target-random
    set number-of-runs number-of-runs - 1
end 

;;; Error Message reported when timeout expires.
to print-timeout-message
   output-print "*** Timeout!! ***"
   wait 0.5
end 


to run-experiment
  place-remove-targets
  wait time-interval
  ifelse Trail [ask ernests [pendown]] [ask ernests [penup]] 
  ask ernests [do]
  tick
end

;; set up the environment (to look the same as the original ernest environment.)
to draw-environment
  create-markers 1 [set shape "grid" set color grey stamp]
  foreach [(list pxcor pycor)] of patches [ask markers [move-to patch first ? item 1 ? stamp]]
  ask markers [die]
  ask patches with [pxcor >= -6 or pycor >= -5] [set pcolor 24.5]
  ask patches with [pxcor = ( - max-pxcor) or pxcor = max-pxcor or pycor = max-pycor or pycor = (- max-pycor)] [set pcolor 14.5]
end

;;; Custom Placement 

to custom-placement
user-message "Once you place your mouse in Ernest's environent you can set the position of enrest by drag and drop and place/remove targets by clicking on them. Move the mouse out of the area to finish."
while [not mouse-inside?] [wait 0.05]
  loop [
  ifelse any? ernests-on patch mouse-xcor round mouse-ycor 
     [move-ernie]
     [place-remove-targets]
     if not mouse-inside? [stop]  
  ]  
end



to rotate-ernie
  ask ernests [rt 90]
end

to move-ernie
  while [mouse-down?] [ask ernests [setxy round mouse-xcor round mouse-ycor]]
end 

to place-remove-targets
  if mouse-down? [every 0.2 [switch-status-targets]]
end  
  

to switch-status-targets
   ifelse any? targets-on patch round mouse-xcor round mouse-ycor
    [ask targets-on patch round mouse-xcor round mouse-ycor [die]]
    [create-targets 1 [set shape "fish 2" set color ##target-color setxy round mouse-xcor round mouse-ycor]]
end
;;; Just placing target with a mouse click. Carefully, since 
;;; without checks it produces a large number of overlapping patches
;;; target is placed in the center of the patch. 

;;; Clearing a target
;to clear-target
; if mouse-down? and any? targets-on patch round mouse-xcor round mouse-ycor
;    [ask targets-on patch round mouse-xcor round mouse-ycor [die]]
;end

;;; clear-all-targets
to clear-all-targets
  ask targets [die]
end

;;; {Place a random target
to place-target-random
  let p clear-patch
  ifelse p = nobody 
  [user-message "Cannot place any more targets. Grid is Full!"]
  [create-targets 1 [set shape "food" set color ##target-color move-to p]]
   end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Main creation of ernest.
to create-ernest
 create-ernests 1 
 [set shape "face happy" 
  set heading 0 
  erny-has-eyes
  move-to clear-patch
  ;; These are locals now in case we would like to include multiple agents.
  set stimuli1 ""
  set config_action "" 
  ;; The imos must be initialized in the context of each agent.
  imos:init 5 4
  imos:interaction ">" "   " step
  imos:interaction "^" "   " turn
  imos:interaction "v" "   " turn
  ]
end 

;;; Now we create the eyes of ernest and tie them to the body.
to erny-has-eyes 
  hatch-eyes 1 [setup-eyes -45 create-left-eye-from myself [tie]]
  hatch-eyes 1 [setup-eyes 45 create-right-eye-from myself [tie]]
end 

; setup the "eyes" of Ernie...
to setup-eyes [angle]
  set shape "ernest-eye" 
  set heading angle 
  set color white 
  set previously-seen false ;; nothing seen
  set previously-smelled false ;;nothing smelled
  set distance-of-target 1000 ;; nothing seen
  end


;; Symbols Actuators Sensors Description Intrinsic  satisfaction
;;  ^    (^) Turn left True Turn 90° left toward adjacent empty square 0    (indifferent)
;;       [^] False Turn 90° left toward adjacent wall -5   (dislike)
;;  >    (>) Forward True Move forward 0     (indifferent)
;;       [>] False Bump wall -8   (dislike)
;;  v    (v) Turn right True Turn 90° right toward adjacent empty square 0     (indifferent)
;;       [v] False Turn 90° right toward adjacent wall -5    (dislike)
;;       * Appear Target appears in distal sensor field 15   (love)
;;       + Closer Target approaches in distal sensor field 10   (enjoy)
;;       x Reached Target reached according to distal sensor 15   (love)
;;       o  Disappear Target disappears from distal sensor field -15  (hate)

;;; ernest actions
to-report move
  ifelse wall-ahead 
   [ask patch-ahead 1 [pflash] report "w"] 
   [move-to patch-ahead 1 report " "]
  ;;stamp
end 

;to-report touch
;  ifelse wall-ahead 
;    [report "w"]
;    [report " "] 
;end 

to-report turn-left
  lt 90
  ifelse wall-ahead 
    [report "w"]
    [report " "]
end

to-report turn-right
  rt 90
  ifelse wall-ahead 
    [report "w"]
    [report " "]
end


;;; Is there a wall ahead?
to-report wall-ahead
  report [pcolor = 14.5] of patch-ahead 1 
end


;; The visual component is probably the most complicated to implement in reactive terms.
;;       * Appear Target appears in distal sensor field 15   (love)
;;       + Closer Target approaches in distal sensor field 10   (enjoy)
;;       x Reached Target reached according to distal sensor 15   (love)
;;       o  Disappear Target disappears from distal sensor field -15  (hate)

;;; reporter see - This is not at all a nice code. Needs re-engineeering.  
to-report see
  
  if internal_state = "hungry"
  [let target-in-sight one-of targets in-cone  100 90
  if target-in-sight != nobody and distance target-in-sight < 1 [report "x"] ;; reached
  ifelse previously-seen 
    ;; seen something before
    [ifelse target-in-sight = nobody 
      [set color white set previously-seen false set distance-of-target 1000 report "o"] ;; lost target 
      [ifelse distance target-in-sight < distance-of-target [set distance-of-target distance target-in-sight report "+"] [report " "]  ] ;; closer or same
      ]
   ;; seen nothing before
   [ifelse target-in-sight = nobody 
      [report " "]  ;; seen nothing have seen nothing
      [set color ##target-color  set previously-seen true set distance-of-target distance target-in-sight report "*"] ;; appear
     ]]
    
end   


to-report split [ #string #sep ] ; #sep must be non-empty string
  let result [] ; return value
  let w length #sep
  loop ; exit when done
  [ let next-pos position #sep #string
    if not is-number? next-pos
    [ report reverse (fput #string result) ]
    set result fput (substring #string 0 next-pos) result
    set #string substring #string (next-pos + w) (length #string)
  ]
end

;;; Should be organized with subroutines.
to do
  ;; the action
 let action imos:step  stimuli1 satisfaction

 if first action = ">"
  [set status move]
 if first action = "^"
  [set status turn-left]
 if first action = "v"
  [set status turn-right]
 if any? targets-here [grab-target stop]
  
  ;; the stimuli
 let leftStimulus first [see] of out-left-eye-neighbors 
 let rightStimulus first [see] of out-right-eye-neighbors


 ;;ifelse leftStimulus = " " and rightStimulus = " "
 ;; [set stimuli1 ""] ;; needs to be compliant with the interactions defined in the setup
 set stimuli1 word leftStimulus  rightStimulus
  
 ;; the satisfaction
 ifelse first action = ">" 
  [ifelse status = " "
    [set satisfaction step] 
    [set satisfaction bump]]
  [set  satisfaction turn]
  
  
 if leftStimulus = "x" [set satisfaction satisfaction + appear ]
 if leftStimulus = "+" [set satisfaction satisfaction + closer]
 if leftStimulus = "*" [set satisfaction satisfaction + appear]
 if leftStimulus = "o" [set satisfaction satisfaction + disappear]
 if rightStimulus = "x" [set satisfaction satisfaction + appear]
 if rightStimulus = "+" [set satisfaction satisfaction + closer]
 if rightStimulus = "*" [set satisfaction satisfaction + appear]
 if rightStimulus = "o" [set satisfaction satisfaction + disappear]
 if stimuli1 = "oo" [set satisfaction -10] ;;; this helps
 
 
 set stimuli1 word status stimuli1

 
 

 
;; Defining internal states for Ernest
 
 if internal_state = "hungry" [set config_action word "hungry" stimuli1]
  

 if status = "w" or status = "woo" [ 
   set config "hitwall"]
 
 if config_action = "hungry *" or config_action = "hungry  *" or config_action = "hungry **" [
  set config "smellfood"]
 
 if config_action = "hungry +" or config_action = "hungry  +" or config_action = "hungry ++" [
   set config "gofood"]
 
 if config_action = "hungry o" or config_action = "hungry  o" or config_action = "hungry oo" [
   set config "eatfood"]
 
 output-print config
 
;;Building the training vector matrix   

 if status = "w" or status = "woo" [
   let temp item random 25 list_hit
   let word_length length words
    let sentence_length length sentences
    let temp1 split temp " "
    let col []
    foreach temp1 [
      let flag 0
      let #temp1 ?
      foreach words [
        let #words ?
        if #temp1 = #words [
          set flag 1
        ]
      ]
      if flag = 0[
        set words lput #temp1 words
        set word_length word_length + 1
      ]
    ]
    
    let t 0
    foreach words [
      let flag 0
      let #words ?
      foreach temp1 [
        let #temp1 ?
        if #temp1 = #words [
          set flag 1
          set col lput 1 col
          set t t + 1
        ] 
      ]
      if flag = 0 [
        set col lput 0 col
        set t t + 1
      ]
    ]
    while [t < 250 ] [
       set col lput 0 col
       set t t + 1
     ]
    
    set sentences lput temp sentences
    matrix:set-column mat sentence_length col
    
    set mat matrix:set-and-report mat 0 sentence_length 1 
    
   
   ]

  if config_action = "hungry * " or config_action = "hungry  *" or config_action = "hungry **" [
   let temp item random 25 list_seefood
    let word_length length words
    let sentence_length length sentences
    let temp1 split temp " "
    let col []
    foreach temp1 [
      let flag 0
      let #temp1 ?
      foreach words [
        let #words ?
        if #temp1 = #words [
          set flag 1
        ]
      ]
      if flag = 0[
        set words lput #temp1 words
        set word_length word_length + 1
      ]
    ]
    
    let t 0
    foreach words [
      let flag 0
      let #words ?
      foreach temp1 [
        let #temp1 ?
        if #temp1 = #words [
          set flag 1
          set col lput 1 col
          set t t + 1
        ] 
      ]
      if flag = 0 [
        set col lput 0 col
        set t t + 1
      ]
    ]
    while [t < 250 ] [
       set col lput 0 col
       set t t + 1
     ]
    
    set sentences lput temp sentences
    matrix:set-column mat sentence_length col
    
    set mat matrix:set-and-report mat 1 sentence_length 1 
  ]

 
  if config_action = "hungry + " or config_action = "hungry ++" or config_action = "hungry  +" [
   let temp item random 25 list_gofood
     let word_length length words
    let sentence_length length sentences
    let temp1 split temp " "
    let col []
    foreach temp1 [
      let flag 0
      let #temp1 ?
      foreach words [
        let #words ?
        if #temp1 = #words [
          set flag 1
        ]
      ]
      if flag = 0[
        set words lput #temp1 words
        set word_length word_length + 1
      ]
    ]
    
    let t 0
    foreach words [
      let flag 0
      let #words ?
      foreach temp1 [
        let #temp1 ?
        if #temp1 = #words [
          set flag 1
          set col lput 1 col
          set t t + 1
        ] 
      ]
      if flag = 0 [
        set col lput 0 col
        set t t + 1
      ]
    ]
    while [t < 250 ] [
       set col lput 0 col
       set t t + 1
     ]
    
    set sentences lput temp sentences
    matrix:set-column mat sentence_length col
    
    set mat matrix:set-and-report mat 2 sentence_length 1 
    
     ]  

if config_action = "hungry o " or config_action = "hungry oo" or config_action = "hungry  o" [
   let temp item random 25 list_eatfood
     let word_length length words
    let sentence_length length sentences
    let temp1 split temp " "
    let col []
    foreach temp1 [
      let flag 0
      let #temp1 ?
      foreach words [
        let #words ?
        if #temp1 = #words [
          set flag 1
        ]
      ]
      if flag = 0[
        set words lput #temp1 words
        set word_length word_length + 1
      ]
    ]
    
    let t 0
    foreach words [
      let flag 0
      let #words ?
      foreach temp1 [
        let #temp1 ?
        if #temp1 = #words [
          set flag 1
          set col lput 1 col
          set t t + 1
        ] 
      ]
      if flag = 0 [
        set col lput 0 col
        set t t + 1
      ]
    ]
    while [t < 250 ] [
       set col lput 0 col
       set t t + 1
     ]
    
    set sentences lput temp sentences
    matrix:set-column mat sentence_length col
    
    set mat matrix:set-and-report mat 3 sentence_length 1 
   
  ]  
  

end 

;;;Training the Hopfield Network

to learn
  
let b 0

let temp []
let temp1 0
let i 0
let j 0

 while [b < 220] [
   
     set temp matrix:get-column mat b
  
set j 0   
     
     while [j < 250] [
       set i 0
       while [i < 250] [
         
         if i != j [
         set temp1 matrix:get weights i j
         ifelse (item i temp != item j temp)
         [set weights matrix:set-and-report weights i j (temp1 - 1)]
         [set weights matrix:set-and-report weights i j (temp1 + 1)]
         ]
         set i (i + 1)  
         ]
        set j (j + 1)
        
               ] 
     set b (b + 1) 
     show b
      ]  
end 


;;; Updating he weight matrix to test for activations 

to test
 
  let input matrix:make-constant 250 1 0
  set input matrix:set-and-report input 158 0 1
;  set input matrix:set-and-report input 3 0 1 
  set output matrix:times weights input  
  
  let index matrix:make-constant 250 1 0
  
  let i 0
  let j 0 
 let boolean 1 
  while [i < 250][
    set index matrix:set-and-report index i 0 i
    set i i + 1]
  
  
  set i 2
  set j 1
  
  print output
  
  while [i < 250][
    
    set boolean 1 
    
      while [((boolean = 1) and (matrix:get output j 0) < (matrix:get output (j - 1) 0)) ][
      
      let temp1 matrix:get output j 0
      matrix:set output j 0 matrix:get output (j - 1) 0
      matrix:set output (j - 1) 0 temp1
      
      let temp2 matrix:get index j 0
      matrix:set index j 0 matrix:get index (j - 1) 0
      matrix:set index (j - 1) 0 temp2
      
     set j (j - 1)
      
      ifelse j > 0 [set boolean 1] [set boolean 0]  
   ]
    
    set j i 
    set i (i + 1)
       
  ]

 let a length words
 
 while [a < 250]
 [set words lput "null" words 
   set a (a + 1)]
 
 set i (length words - 1)
   
  while [i > 0]
 [let temp3 matrix:get index i 0
   let temp4 matrix:get output i 0 
 print word temp4 item temp3 words
   set i (i - 1)
   ]
  
end


to grab-target 
  ask targets-here [die]
end


;;; Utilities 
to-report clear-patch 
  report one-of patches with [pcolor != 14.5 and not any? turtles-here ]
end

;;; Utility to make a patch flash for a fragment of a second.
to pflash 
  let old-color pcolor
  set pcolor red
  wait 0.2
  set pcolor old-color
end




  



@#$#@#$#@
GRAPHICS-WINDOW
7
10
407
371
6
5
30.0
1
10
1
1
1
0
0
0
1
-6
6
-5
5
0
0
1
ticks

BUTTON
662
87
831
120
Re-initialise Agent
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
432
23
525
56
Run
run-experiment\n
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
533
23
624
56
Step
run-experiment
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

OUTPUT
668
331
835
516
12

SLIDER
630
23
797
56
time-interval
time-interval
0
1
0.05
0.05
1
NIL
HORIZONTAL

SWITCH
434
232
527
265
Trail
Trail
1
1
-1000

BUTTON
533
232
642
265
Clear Trails
clear-drawing draw-environment
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
28
403
256
436
number-of-runs
number-of-runs
0
100
0
1
1
NIL
HORIZONTAL

SLIDER
28
438
256
471
timeout
timeout
200
2000
210
10
1
NIL
HORIZONTAL

BUTTON
28
476
257
509
Run a Series of Runs
safe-run-a-series
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
532
87
641
120
appear
appear
-20
20
15
1
1
NIL
HORIZONTAL

SLIDER
532
129
642
162
closer
closer
-20
20
10
1
1
NIL
HORIZONTAL

SLIDER
532
170
641
203
disappear
disappear
-20
20
-15
1
1
NIL
HORIZONTAL

SLIDER
433
128
525
161
bump
bump
-20
20
-8
1
1
NIL
HORIZONTAL

BUTTON
661
128
832
161
Reset Values to Default
default
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
432
87
524
120
step
step
-20
20
-1
1
1
NIL
HORIZONTAL

SLIDER
433
169
525
202
turn
turn
-20
20
0
1
1
NIL
HORIZONTAL

TEXTBOX
666
291
850
332
Enacted Interaction / Satisfaction\n\n
12
103.0
1

TEXTBOX
10
380
183
400
Batch Experiment Area
14
12.0
1

TEXTBOX
433
277
583
295
Targets
12
0.0
1

TEXTBOX
434
210
584
228
Trails
12
0.0
1

TEXTBOX
432
65
582
83
Satisfaction Values
12
0.0
1

TEXTBOX
432
420
531
449
Learning The
14
102.0
1

BUTTON
663
191
833
224
Place Agent and Targets
custom-placement
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
663
231
833
264
Rotate Agent
rotate-ernie
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

TEXTBOX
665
174
815
192
Placing Ernest
12
0.0
1

BUTTON
481
347
601
380
Remove all food
clear-all-targets
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

INPUTBOX
393
412
636
472
internal_state
hungry
1
0
String

BUTTON
508
310
572
343
Food!
place-target-random
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This model demonstrates an agent that is environmentally agnostic and intrinsically motivated.

Environmentally agnostic means that the agent's decisional process was implemented with no preconception of the environment. There is not even any parameter specifying that the agent operates in a two dimensional space.

Intrinsically motivated means that the agent's decisional process was implemented with no preconception of a specific task or a specific strategy to perform. Instead, the agent follows intrinsic drives. 

The nuance between a predefined task and intrinsic drives is subtle. This models seeks to clarify this nuance.

HOW IT WORKS
------------
The agent has predefined possibilities of interaction with the environment. 

On each step, the agent can choose between three primitive actions: move one step forward, turn 90 degrees to the right, or turn 90 degrees to the left. 

Primitive actions result in primitive feedback: bump into a wall, target appears in visual field, target enlarges in visual field (got closer), target disappears from visual field (each eye has a 90 degree visual span). 

Primitive actions associated with primitive feedback are called primitive interactions. Primitive interactions have values that you can predefine. 

The agent's algorithm tries to perfom primitive interactions with high values and to avoid primitive interactions with negative values. You can, therefore, understand these values as the agent's likeness (positive values) or dislike (negative values) of each primitive interaction.

In fact the model provides an ouput window that shows the interactions enacted by the agent and the satisfaction it received by performing these in the environment. 

HOW TO USE IT
-------------
Click Run to run the agent.

Click the grid to insert a target.

Click Re-initialize to reset the agent's memory and restart the learning process from scratch.

You can use the cursors to specify the value of each aspect of interactions:
* Step: the value of moving one step forward.
* Bump: the value of bumping into a wall.
* Turn: the value of turning 90 degrees (left or right).
* Appear: the value of a target appearing in an eye's visual field.
* Closer: the value of a target enlarging in an eye's visual field.
* Disappear: the value of all targets disappearing from an eye's visual field. 
The values of step, bump, turn are summed with the values associated with each eye to form the total primitive interaction's value.

Click Re-initialize to apply the new values and clear and restart the learning from scratch.

Click Reset Values to Default to reset the values to default, indeed.

Batch Experiment Area:
This set of controls allows to run the agent multiple times. Each run leaves a trail behind to see behavioral patterns. Timeout parameter controls the maximum time (in ticks) the agent has to get to the target (This makes the agent get bored when it is cought in an infinite loop). Note that it does not reinitializes the agent, so you can train the agent and then see how it behaves.


THINGS TO NOTICE
----------------
Notice how the values of interactions impact the agent's behavior:

* Give positive values for stepping and for bumping, and all other values negative (click Re-initialize to apply your changes). Notice that the agent will move forward and keep bumping a wall. Of course, he loves that!

* Give positive values for turning, and all other values negative. Notice that the agent will keep spinning in place. Now, he loves that!

Notice that the agent is deterministic:

* Use the Place Agent and Targets button to define a specific initial configuration. Click the Re-initialize button. Click Run. Repeat these operations with the same initial configuration. Notice that the agent behaves the same in each run (until you introduce variation by inserting targets in different places or at different times).

Notice that the agent learns different strategies depending only on its early experience.

* Click the Diagonal Behavior button. Click Run. After the agent ate the first target, introduce new targets by clicking on the grid. Notice that the agent has learned a behavior consisting of reaching the target in a diagonal stair-step progression.
 
* Click the Tangential Behavior button. Click Run. After the agent ate the first target, introduce new targets by clicking on the grid. Notice that the agent has learned another behavior that we call the tangential strategy. The tangential strategy results from that the targets disappears from the visual field only when the agent has gone one step too far. The agent thus needs to go back one step to align itself witht the target.

(Notice that the Diagonal Behavior button and the Tangential Behavior button reset the values to default. This shows that "twin" agents (agents with identical inborn parameters) that have different experiences during their "youth" will behave differently when they are "grown up". Also, this demonstrates that the strategies are not pre-encoded in the agent but learned).

THINGS TO TRY
-------------
Try to generate new behaviors by giving different values to interactions. For example, try to make an agent that moves away from targets. 


EXTENDING THE MODEL
-------------------
* Define other interaction, for example: turning 45degrees and possibly moving in diagonal.

* Define different kinds of targets and make an agent that alternatively seeks targets of each kind.

* Introduce several agents.


NETLOGO FEATURES
----------------
This model implements the IMOS extension (Intrinsic Motivation System). 


RELATED MODELS
--------------
There is no related NetLogo model that we would be aware of.


CREDITS AND REFERENCES
----------------------
This model was implemented by Ilias Sakellariou (University of Macedonia, Greece), using the IMOS NetLogo extension implemented by Olivier Georgeon (Universite de Lyon / CNRS, France).
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

caterpillar
true
0
Polygon -7500403 true true 165 210 165 225 135 255 105 270 90 270 75 255 75 240 90 210 120 195 135 165 165 135 165 105 150 75 150 60 135 60 120 45 120 30 135 15 150 15 180 30 180 45 195 45 210 60 225 105 225 135 210 150 210 165 195 195 180 210
Line -16777216 false 135 255 90 210
Line -16777216 false 165 225 120 195
Line -16777216 false 135 165 180 210
Line -16777216 false 150 150 201 186
Line -16777216 false 165 135 210 150
Line -16777216 false 165 120 225 120
Line -16777216 false 165 106 221 90
Line -16777216 false 157 91 210 60
Line -16777216 false 150 60 180 45
Line -16777216 false 120 30 96 26
Line -16777216 false 124 0 135 15

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

drop
false
0
Circle -7500403 true true 73 133 152
Polygon -7500403 true true 219 181 205 152 185 120 174 95 163 64 156 37 149 7 147 166
Polygon -7500403 true true 79 182 95 152 115 120 126 95 137 64 144 37 150 6 154 165

ernest-body
true
0
Circle -6459832 true false 90 135 120
Circle -955883 true false 75 60 150
Polygon -6459832 true false 135 225 150 270 165 225 150 195
Line -7500403 true 105 150 90 180
Line -7500403 true 105 150 195 150
Line -7500403 true 195 150 210 180

ernest-eye
true
0
Polygon -7500403 true true 75 60 90 45 105 30 135 15 165 15 195 30 210 45 150 105 90 45

ernest-left-eye
true
0
Polygon -7500403 true true 15 105 120 105 120 15 105 15 75 30 60 45 45 60 30 75

ernest-right-eye
true
0
Polygon -7500403 true true 180 15 180 105 285 105 270 75 255 60 240 45 225 30 195 15

ernest-target
true
0
Circle -7500403 true true 58 58 182

face happy
true
8
Circle -7500403 true false 7 8 285
Circle -16777216 true false 180 75 60
Circle -16777216 true false 60 75 60
Polygon -16777216 true false 150 255 210 239 238 213 253 191 233 179 210 203 191 218 150 225 108 218 90 203 73 181 49 194 64 217 88 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

fish 2
false
0
Polygon -1 true false 56 133 34 127 12 105 21 126 23 146 16 163 10 194 32 177 55 173
Polygon -7500403 true true 156 229 118 242 67 248 37 248 51 222 49 168
Polygon -7500403 true true 30 60 45 75 60 105 50 136 150 53 89 56
Polygon -7500403 true true 50 132 146 52 241 72 268 119 291 147 271 156 291 164 264 208 211 239 148 231 48 177
Circle -1 true false 237 116 30
Circle -16777216 true false 241 127 12
Polygon -1 true false 159 228 160 294 182 281 206 236
Polygon -7500403 true true 102 189 109 203
Polygon -1 true false 215 182 181 192 171 177 169 164 152 142 154 123 170 119 223 163
Line -16777216 false 240 77 162 71
Line -16777216 false 164 71 98 78
Line -16777216 false 96 79 62 105
Line -16777216 false 50 179 88 217
Line -16777216 false 88 217 149 230

fish 3
false
0
Polygon -7500403 true true 137 105 124 83 103 76 77 75 53 104 47 136
Polygon -7500403 true true 226 194 223 229 207 243 178 237 169 203 167 175
Polygon -7500403 true true 137 195 124 217 103 224 77 225 53 196 47 164
Polygon -7500403 true true 40 123 32 109 16 108 0 130 0 151 7 182 23 190 40 179 47 145
Polygon -7500403 true true 45 120 90 105 195 90 275 120 294 152 285 165 293 171 270 195 210 210 150 210 45 180
Circle -1184463 true false 244 128 26
Circle -16777216 true false 248 135 14
Line -16777216 false 48 121 133 96
Line -16777216 false 48 179 133 204
Polygon -7500403 true true 241 106 241 77 217 71 190 75 167 99 182 125
Line -16777216 false 226 102 158 95
Line -16777216 false 171 208 225 205
Polygon -1 true false 252 111 232 103 213 132 210 165 223 193 229 204 247 201 237 170 236 137
Polygon -1 true false 135 98 140 137 135 204 154 210 167 209 170 176 160 156 163 126 171 117 156 96
Polygon -16777216 true false 192 117 171 118 162 126 158 148 160 165 168 175 188 183 211 186 217 185 206 181 172 171 164 156 166 133 174 121
Polygon -1 true false 40 121 46 147 42 163 37 179 56 178 65 159 67 128 59 116

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

food
false
0
Polygon -7500403 true true 30 105 45 255 105 255 120 105
Rectangle -7500403 true true 15 90 135 105
Polygon -7500403 true true 75 90 105 15 120 15 90 90
Polygon -7500403 true true 135 225 150 240 195 255 225 255 270 240 285 225 150 225
Polygon -7500403 true true 135 180 150 165 195 150 225 150 270 165 285 180 150 180
Rectangle -7500403 true true 135 195 285 210

grid
false
0
Rectangle -1 false false 0 0 300 300

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

person lumberjack
false
0
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -2674135 true false 60 196 90 211 114 155 120 196 180 196 187 158 210 211 240 196 195 91 165 91 150 106 150 135 135 91 105 91
Circle -7500403 true true 110 5 80
Rectangle -7500403 true true 127 79 172 94
Polygon -6459832 true false 174 90 181 90 180 195 165 195
Polygon -13345367 true false 180 195 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285
Polygon -6459832 true false 126 90 119 90 120 195 135 195
Rectangle -6459832 true false 45 180 255 195
Polygon -16777216 true false 255 165 255 195 240 225 255 240 285 240 300 225 285 195 285 165
Line -16777216 false 135 165 165 165
Line -16777216 false 135 135 165 135
Line -16777216 false 90 135 120 135
Line -16777216 false 105 120 120 120
Line -16777216 false 180 120 195 120
Line -16777216 false 180 135 210 135
Line -16777216 false 90 150 105 165
Line -16777216 false 225 165 210 180
Line -16777216 false 75 165 90 180
Line -16777216 false 210 150 195 165
Line -16777216 false 180 105 210 180
Line -16777216 false 120 105 90 180
Line -16777216 false 150 135 150 165
Polygon -2674135 true false 100 30 104 44 189 24 185 10 173 10 166 1 138 -1 111 3 109 28

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
